<?php
namespace FM\Service;

use FM\Factory\SolicitacaoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FileServerService;

class SolicitacaoService {
    
    protected $SolicitacaoFactory;
    protected $environment;
    protected $newCode;
    protected $connection;
    protected $FileServerService;
    protected $fileServerApiKeys;
    
    public function __construct (SolicitacaoFactory $SolicitacaoFactory, Environment $Environment, NewCode $newCode, EntityManager $entityManager, FileServerService $FileServerService, $fileServerApiKeys) {
        $this->SolicitacaoFactory           = $SolicitacaoFactory;
        $this->environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->connection                   = $entityManager->getConnection(); 
        $this->FileServerService = $FileServerService;
        $this->fileServerApiKeys = $fileServerApiKeys;
    }
    
    // Busca as solicitações
    public function getRequests(){
        $requestsList = $this->SolicitacaoFactory->getRequests();
        return $requestsList;
    }
    
    // Busca os tipos de solicitação
    public function getRequestTypes(){
        $typesList = $this->SolicitacaoFactory->getRequestTypes();
        return $typesList;
    }
    
   // Busca as informações de uma solicitação específica
    public function getRequestInfo($nrsolicitacao){
        $requestInfo = $this->SolicitacaoFactory->getRequestInfo($nrsolicitacao);
        return $requestInfo;
    }
    
    // Busca o histórico de uma solicitação
    public function getRequestHistoric($nrsolicitacao){
        $requestInfo = $this->SolicitacaoFactory->getRequestHistoric($nrsolicitacao);
        return $requestInfo;
    }
    
    // Busca todas as hierarquias
    public function insertRequestType($params){
        $nrtiposolicitacao  = $this->newCode->getCodeByTableName('FCM_TIPOSOLICITACAO', $this->environment->getNrOrg(), 1, 10, true);
        
        $typeParams = array(
            'NRTIPOSOLICITACAO' => $nrtiposolicitacao,
            'NMTIPOSOLICITACAO' => $params['NMTIPOSOLICITACAO'],
            'IDPRIORIDADEPADR' => $params['IDPRIORIDADEPADR'],
            'NRORG' => $this->environment->getNrOrg()
        );

        try {
            $this->SolicitacaoFactory->insertRequestType($typeParams);
            $response = "Inserido com sucesso";
        } catch (\Exception $e) {
            $response = $e->getMessage();
        }

        return $response;
    }
    
    // Processo de clasificação -> Alteração nas tabelas de solicitação/sol_tarefas, checklistAgend e  histórico
    public function classifyRequest($params){
 
        $requestParams = array(
            'NRSOLICITACAO' => $params['NRSOLICITACAO'],
            'NMSOLICITACAO' => $params['NMSOLICITACAO'],
            'IDSITUACAO' => '2',
            'NRTIPOSOLICITACAO' => $params['NRTIPOSOLICITACAO'],
            'IDPRIORIDADE' => $params['IDPRIORIDADE'],
            'NRFUNCRESPONSAVEL' => $params['NRFUNCRESPONSAVEL'],
            'DSSOLICITACAO' => $params['DSSOLICITACAO']
        );
 
        $nrChecklistAgend  = $this->newCode->getCodeByTableName('FCM_CHECKLISTAGEND', $this->environment->getNrOrg(), 1, 10, true);       
        $checklistParams = array(
            'NRCHECKLISTAGEND' => $nrChecklistAgend,
            'NRLOCALCLIENTE' => $params['NRLOCALCLIENTE'],
            'NRCHECKLIST' => $params['NRCHECKLIST'],
            'NRFUNCIONARIO' => $params['NRFUNCRESPONSAVEL'],
            'NRORG' => $this->environment->getNrOrg(),
            'NMCHECKLISTAGEND' => $params['NMSOLICITACAO']
        );
        
        $nrSolicitacaoTarefas  = $this->newCode->getCodeByTableName('FCM_SOLICITACAOTAREFAS', $this->environment->getNrOrg(), 1, 10, true);    
        $taskRequestParams = array(
            'NRSOLICITACAOTAREFAS' => $nrSolicitacaoTarefas,
            'NRSOLICITACAO' => $params['NRSOLICITACAO'],
            'NRCHECKLISTAGEND' => $nrChecklistAgend,
            'NRORG' => $this->environment->getNrOrg()
        );
        
        $nrSolicitacaoHistorico  = $this->newCode->getCodeByTableName('FCM_SOLICITACAOHISTORICO', $this->environment->getNrOrg(), 1, 10, true);    
        $historicParams = array(
            'NRSOLICITACAOHISTORICO' => $nrSolicitacaoHistorico,
            'NRSOLICITACAO' => $params['NRSOLICITACAO'],
            'NMSOLICITACAO' => $params['NMSOLICITACAO'],
            'DTHISTSOLICITACAO' => $params['DTHISTSOLICITACAO'],
            'IDSITUACAO' => '2',
            'IDTIPOHISTORICO' => '2',
            'NRFUNCSOLICITANTE' => $params['NRFUNCRESPONSAVEL'],
            'DSSOLICITACAO' => $params['DSSOLICITACAO'],
            'NRORG' => $this->environment->getNrOrg(),
        );
        // debugApi([$historicParams]);

        $this->connection->beginTransaction();
        try {
            $this->SolicitacaoFactory->updateRequest($requestParams);
            $this->SolicitacaoFactory->createTask($checklistParams); // Cria uma tarefa
            $this->SolicitacaoFactory->createTaskRequest($taskRequestParams); // Cria uma relação de tarefa com solicitação
            $this->SolicitacaoFactory->insertHistoricData($historicParams);
            
            $this->connection->commit();
            $response = "Classificado com sucesso";
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response = $e->getMessage();
        }

        return $response;
    }
    
     // Edita o tipo de solicitação
    public function updateRequestType($params){
        $typeParams = array(
            'NRTIPOSOLICITACAO' => $params['NRTIPOSOLICITACAO'],
            'NMTIPOSOLICITACAO' => $params['NMTIPOSOLICITACAO'],
            'IDPRIORIDADEPADR' => $params['IDPRIORIDADEPADR']
        );
        
        try {
            $this->SolicitacaoFactory->updateRequestType($typeParams);
            $response = "Editado com sucesso";
        } catch (\Exception $e) {
            $response = $e->getMessage();
        }

        return $response;
    }
    
    public function getLocalRequest($params) {
        $locals = $this->SolicitacaoFactory->getLocalRequest($params);
        return $locals;
    }
    
    public function setNewRequest($parametros) {
        $this->connection->beginTransaction();

         try {
                $NRORG = $this->environment->getNrOrg();
                $NRSOLICITACAO = $this->newCode->getCodeByTableName("FCM_SOLICITACAO", $NRORG, 1, 10, true);
                $params = array(
                    "NRSOLICITACAO"  => $NRSOLICITACAO,
                    "NMSOLICITACAO"  => $parametros['NMSOLICITACAO'],
                    "IDSITUACAO"     => $parametros['IDSITUACAO'],
                    "NRLOCALCLIENTE" => $parametros['NRLOCALCLIENTE'],
                    "NRFUNCABERTURA" => $parametros['NRFUNCABERTURA'],
                    "DSSOLICITACAO"  => $parametros['DSSOLICITACAO'],
                    "NRORG"          => $NRORG
                );
                $this->SolicitacaoFactory->insertNewRequest($params);

                $NRSOLICITACAOHISTORICO = $this->newCode->getCodeByTableName("FCM_SOLICITACAOHISTORICO", $NRORG, 1, 10, true);

                $params = array(
                    "NRSOLICITACAOHISTORICO" => $NRSOLICITACAOHISTORICO,
                    "NRSOLICITACAO"          => $NRSOLICITACAO,
                    "IDSITUACAO"             => 1,
                    "IDTIPOHISTORICO"        => 1,
                    "NRFUNCSOLICITANTE"      => $parametros['NRFUNCABERTURA'],
                    "DSSOLICITACAO"          => $parametros['DSSOLICITACAO'],
                    "IDSATISFACAOSOLICITA"   => null,
                    "NRORG"                  => $NRORG
                );
                

                $this->SolicitacaoFactory->insertRequestHistory($params);

                $row = $this->FileServerService->uploadImagensManut($parametros['IMAGENS']);
                foreach ($row as $file) {
                    $this->SolicitacaoFactory->insertImg(array(
                        "NRIMGSOLICITACAO" => $this->newCode->getCodeByTableName('FCM_IMGSOLICITACAO', $this->environment->getNrOrg(), 1, 10, true),
                        "NRSOLICITACAO" => $NRSOLICITACAO,
                        "DSCAMINHO" => $file['DS'],
                        "NRORG" => $NRORG,
                        "DSTYPE" => $file['DSTYPE'],
                        "NRSIZE" => $file['QTSIZE'],
                        "CDOPERINCLUSAO" => $this->environment->getCdOperador()
                    ));
                }

            $this->connection->commit();
            $response = "OK";

        } catch(\Exception $e) {

             $this->connection->rollback();
             $response = ($e->getMessage());
        }
        return $response;
    }

    public function getCardsRequests($parametro, $parametroIds) {

        $param = '';

        if ($parametro['selectName'] != '' && $parametro['selectName'] != null) {
            $param = $param." AND S.NRSOLICITACAO = ".$parametro['selectName'];
        }

        if ($parametro['selectLocal'] != '' && $parametro['selectLocal'] != null) {
            $param = $param." AND S.NRLOCALCLIENTE = ".$parametro['selectLocal'];
        }

        if ($parametro['selectSituation'] != '' && $parametro['selectSituation'] != null) {
            $param = $param." AND S.IDSITUACAO = ".$parametro['selectSituation'];
        }

        if ($parametro['TO_CHAR(S.DTSOLICITACAO,\'DD/MM/YYYY\')'] != '' && $parametro['TO_CHAR(S.DTSOLICITACAO,\'DD/MM/YYYY\')'] != null && strlen($parametro['TO_CHAR(S.DTSOLICITACAO,\'DD/MM/YYYY\')']) == 21) {
            $param = $param." AND TO_CHAR(S.DTSOLICITACAO,'DD/MM/YYYY') = ".$parametro['TO_CHAR(S.DTSOLICITACAO,\'DD/MM/YYYY\')'];
        }

        $usersIds = implode(',',$parametroIds); // Transforma o array em uma string com os ids

        $cards = $this->SolicitacaoFactory->getCardsRequests($param, $usersIds);
        return $cards;
    }

    public function showRequest($params) {

        $parametros = array(
            "NRSOLICITACAO"    => $params['NRSOLICITACAO'],
        );

        $details = $this->SolicitacaoFactory->showRequest($parametros);
        return $details;
    }

    public function getSituationRequest() {

        $situation = $this->SolicitacaoFactory->getSituationRequest();
        return $situation;
    }

    public function getNameRequest() {

        $nameRequest = $this->SolicitacaoFactory->getNameRequest();
        return $nameRequest;
    }

    public function evaluateRequest($parametros) {
        $this->connection->beginTransaction();

         try {
            $NRORG = $this->environment->getNrOrg();
            $params = array(
                "NRSOLICITACAO"        => $parametros['NRSOLICITACAO'],
                "IDSITUACAO"           => ($parametros['CONDITION']) ? 4 : 1,
                "DSSOLICITACAO"        => $parametros['DSSOLICITACAO']
            );
            $this->SolicitacaoFactory->upRequest($params);
            
            $NRSOLICITACAOHISTORICO = $this->newCode->getCodeByTableName("FCM_SOLICITACAOHISTORICO", $NRORG, 1, 10, true);

            $params = array(
                    "NRSOLICITACAOHISTORICO" => $NRSOLICITACAOHISTORICO,
                    "NRSOLICITACAO"          => $parametros['NRSOLICITACAO'],
                    "IDSITUACAO"             => ($parametros['CONDITION']) ? 4 : 1,
                    "IDTIPOHISTORICO"        => 4,
                    "NRFUNCSOLICITANTE"      => $parametros['NRFUNCABERTURA'],
                    "DSSOLICITACAO"          => $parametros['DSSOLICITACAO'],
                    "IDSATISFACAOSOLICITA"   => $parametros['IDSATISFACAOSOLICITA'],
                    "NRORG"                  => $NRORG
                );
            $this->SolicitacaoFactory->insertRequestHistory($params);

            $this->connection->commit();
            $response = "OK";

        } catch(\Exception $e) {

             $this->connection->rollback();
             $response = ($e->getMessage());
        }
        return $response;
    }
}
?>